//
//  helper_functions.c
//  Daisyworld
//
//  Created by Christoph Bockhahn & Max Hänze on 26.06.12.
//  Copyright (c) 2012 /. All rights reserved.
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "declarations.h"
#include "mpi_routines.h"
#include <string.h>

int initializeWorld (int x,int y) {
    dumpStep=0;

    alpha = DT/C * DELTA_T / pow(DELTA_X,2);
    normS = 1.0;
    //printf("ALPHA: %f \n \n \n",alpha);
    // Allocate memory
    world =(WorldCell*) malloc(sizeof(WorldCell)*x*y);
    newWorld = (WorldCell*)malloc(sizeof(WorldCell)*x*y);
    // Alle Zellen auf initCell setzen
    for (int i=0; i<x*y;i++){
        world[i].T = TOPT;
        world[i].alive = rand() % 2;
        world[i].A = rand() / (double)RAND_MAX;
    }
#ifdef DEBUG_LOG
    // gebe Gesamtspeicher der Welt aus
    printf("Memory used by world: %.1f MByte \n", 2.0f*sizeof(WorldCell)*x*y/1000000.f);
#endif

    return 0;
}


int getNeighbour(int index, enum neighbour nextNeighbour)
{
    
    switch (nextNeighbour) {
        case left:
            if(index % DIMENSION_X == 0)  //Zelle liegt am linken Rand --> Zelle am rechten Rand in der selben Zeile zurück geben
            {
                return (index+DIMENSION_X-1); 
            }            
            else 
            {
                return (index-1);
            }
            break;
        case right:
            if((index+1) % DIMENSION_X == 0)  //Zelle liegt am rechten Rand --> Zelle am linken Rand in der selben Zeile zurück geben
            {
                return (index-DIMENSION_X+1); 
            }            
            else 
            {
                return (index+1);
            }
            break;
        case down:
            if((index+DIMENSION_X) >= (DIMENSION_X*DIMENSION_Y)) // Zelle liegt am unteren Rand
            {
                return (index+DIMENSION_X-(DIMENSION_X*DIMENSION_X));
            }
            else 
            {
                return (index+DIMENSION_X);
            }
            break;
        case up:
            if((index-DIMENSION_X) < 0) // Zelle liegt am oberen Rand
            {
                return (index-DIMENSION_X+(DIMENSION_X*DIMENSION_X));
            }
            else 
            {
                return (index-DIMENSION_X);
            }
            break;
    }    
    return 0;
}

//gibt eine Zufallszahl im Intervall [a,b] zurück
double rand_intervall(double a, double b)
{
    return (b-a)*(rand() / (double) RAND_MAX ) + a;
}

int getRandomNextNeighbour(int i)
{
    int randomNumber;
    randomNumber = rand() % 4; //random number between 0 and 4
    return getNeighbour(i, randomNumber);
}

void solveTimestep()
{
    solveT_FTCS();
    solveA();
    return;
}



void solveA()
{
    //iteriere durch alle Zellen der WorldCell
    for(int i=0;i<DIMENSION_X*DIMENSION_Y;i++)
    {
            if(world[i].alive == 1) //Zelle lebt
            {
                applyDeathRate(i);
            }
            else { // Zelle lebt nicht
                applyBirthRate(i); 
            }
            newWorld[i].T=world[i].T;
    }
    swapWorlds();
    return;
}

void solveT_FTCS() 
{
//
//
    for (int i=0; i<DIMENSION_X*DIMENSION_Y;i++)
    {
        newWorld[i].T=alpha*(world[getNeighbour(i,up)].T+world[getNeighbour(i,down)].T+world[getNeighbour(i,left)].T+world[getNeighbour(i,right)].T-4*world[i].T);
        newWorld[i].T+=1.0/C*DELTA_T*(normStoS()*(1-world[i].A)-BOLZ*pow(world[i].T,4) );
        newWorld[i].T+=world[i].T;
        newWorld[i].A=world[i].A;
        newWorld[i].alive=world[i].alive;
    }

    swapWorlds();
}



double normStoS()
{
    
    return BOLZ*pow(TOPT,4) / (1.0-A_0)*normS;
}

void swapWorlds()
{
    WorldCell* tmp=world;
    world=newWorld;
    newWorld=tmp;

};


void applyBirthRate(int i)
{
    
    double randomNumber;
    //choose random next neighbour
    int nextNeighbour = getRandomNextNeighbour(i);
    
    if(world[nextNeighbour].alive == 1)
    {                
        //Zelle wächst mit Wsk Beta
        randomNumber = rand_intervall(0,1);  
        if(randomNumber <= beta(world[nextNeighbour].T))
        {
            newWorld[i].alive = 1;
            // Albedo wird von der zufälligen Nachbarzelle übernommen und unterliegt einer Mutationsrate
            newWorld[i].A = mutationFunction(world[nextNeighbour].A);    
        }
    }
    else 
    {
        //Zelle wächst nicht
        newWorld[i].alive = 0; 
        newWorld[i].A = A_0;
    }
}

void applyDeathRate(int i)
{
    double randomNumber = rand_intervall(0,1);
    if(randomNumber < GAMMA)
    {
        newWorld[i].alive = 0; // Zelle stirbt mit Wsk 1-GAMMMA
        newWorld[i].A = A_0; //setze Albedo zurück auf Ausgangswert A_0
        //printf("STIRB! \n");
    }
    else {
        newWorld[i].alive = world[i].alive;
        newWorld[i].A = world[i].A;
    }
}


double mutationFunction(double A)
{
    double result = A + rand_intervall(-MUTATION_RATE, MUTATION_RATE);
    if (result < 0 ) result = 0;
    else if (result > 1) result = 1;
    
    return result;
}


void writeWorldToBinaryFile(WorldCell* worldArray)
{
    char outputName[1000];
    sprintf(outputName,"%s%sWorld_%d.bin",PATH,name,dumpStep);
    FILE* file=fopen(outputName,"wb");
    int nb=DIMENSION_X;
    fwrite(&nb,1,4,file);
    fwrite(worldArray,sizeof(WorldCell),DIMENSION_X*DIMENSION_Y,file);
    
    fclose(file);
    dumpStep++;
}

void writePropertiesToFile(){
    static int firstRun = 1;
   //Ergebnisse ausgeben
    double sum_T=0l;
    double sum_albedo=0l;
    int alive=0;
    double N=DIMENSION_X*DIMENSION_Y;
    for(int i=0;i<N;i++)
    {
        sum_T += fullWorld[i].T;
        sum_albedo += fullWorld[i].A;
        alive += fullWorld[i].alive;
    }
    char outputName[1000];
    sprintf(outputName,"%s%s%s",PATH,name,"stats.txt");
    FILE* file;
    if (firstRun==1){
        file=fopen(outputName, "w");
        firstRun=0;
    }
    else file=fopen(outputName,"a"); 
    
    fprintf(file,"%le %le %le %d\n", normS,sum_T/N,sum_albedo/N,alive);
    fclose(file);
    return;


}

void deallocateAll(){
    free(world);
    free(newWorld);
}



