//
//  mpi_routines.c
//  Daisyworld
//
//  Created by Max Hänze & Christoph Bockhahn on 07.07.12.
//  Copyright (c) 2012 /. All rights reserved.
//
#include "mpi_routines.h"
#include <stdlib.h>
#include <math.h>
void solveTimestepParallel()
{
    solveTParallel_FTCS();
    solveAParallel();
    return;
}

void solveTParallel_FTCS()
{

   for (int i=DIMENSION_X; i<cells+DIMENSION_X;i++)
    {
        newWorld[i].T=alpha*(world[getNeighbour(i,up)].T+world[getNeighbour(i,down)].T);
        newWorld[i].T+=alpha*(world[getNeighbour(i,left)].T+world[getNeighbour(i,right)].T-4*world[i].T);
        newWorld[i].T+=1.0/C*DELTA_T*(normStoS()*(1-world[i].A)-BOLZ*pow(world[i].T,4) );
        newWorld[i].T+=world[i].T;
        newWorld[i].A=world[i].A;
        newWorld[i].alive=world[i].alive;
    }
    swapWorlds();
    communicateWorld();     
}

void solveAParallel()
{
    //iteriere durch alle Zellen der WorldCell
    for(int i=DIMENSION_X;i<cells+DIMENSION_X;i++)
    {
            if(world[i].alive == 1) //Zelle lebt
            {
                applyDeathRate(i);
            }
            else { // Zelle lebt nicht
                applyBirthRate(i); 
            }
            newWorld[i].T=world[i].T;
    }
    swapWorlds();
    communicateWorld();
    return;
}
//Todo write with gather MPI function
void updateFullWorld()
{

    MPI_Status status1;
    if (rank==0){
        for (int i=1;i<totalNodes;i++)
        {
            int start=cells*i;
            MPI_Recv(&fullWorld[start], cells, worldCellType,i, 104, MPI_COMM_WORLD,&status1); 
            
            for (int j=0;j<cells;j++)
                fullWorld[j]=world[DIMENSION_X+j];
        }
    }
    else 
    {
        MPI_Send(&world[DIMENSION_X], cells, worldCellType,0, 104, MPI_COMM_WORLD);
    }
    

}

void communicateWorld()
{
   MPI_Status status[4];
   MPI_Request request[4];
   
   MPI_Isend( &world[cells], DIMENSION_X, worldCellType, neighbor1, 100, MPI_COMM_WORLD, &request[0]);
   MPI_Isend( &world[DIMENSION_X], DIMENSION_X, worldCellType, neighbor2, 101, MPI_COMM_WORLD, &request[1]);
   
   MPI_Irecv( &world[0], DIMENSION_X, worldCellType, neighbor2, 100, MPI_COMM_WORLD ,&request[2]);
   MPI_Irecv( &world[cells+DIMENSION_X], DIMENSION_X, worldCellType, neighbor1, 101, MPI_COMM_WORLD,&request[3]);

   MPI_Waitall (4,request,status);

   /*
   //gerade 1. senden 2. empfangen
   if (rank%2==0)
   {
        MPI_Send(  &world[cells], DIMENSION_X, worldCellType, getProcessNeighbor(1), 100, MPI_COMM_WORLD); 
        MPI_Send(  &world[DIMENSION_X], DIMENSION_X, worldCellType, getProcessNeighbor(-1), 101, MPI_COMM_WORLD);
        
        MPI_Recv( &world[0], DIMENSION_X, worldCellType, getProcessNeighbor(-1), 102, MPI_COMM_WORLD, &status1);
        MPI_Recv( &world[cells+DIMENSION_X], DIMENSION_X, worldCellType, getProcessNeighbor(1), 103, MPI_COMM_WORLD, &status2);
   }

   //ungerade 1. empfangen 2. senden
   else 
   {
        MPI_Recv( &world[0], DIMENSION_X, worldCellType, getProcessNeighbor(-1), 100, MPI_COMM_WORLD, &status1);
        MPI_Recv( &world[cells+DIMENSION_X], DIMENSION_X, worldCellType, getProcessNeighbor(1), 101, MPI_COMM_WORLD, &status2);
        
        MPI_Send(  &world[cells], DIMENSION_X, worldCellType, getProcessNeighbor(1), 102, MPI_COMM_WORLD); 
        MPI_Send(  &world[DIMENSION_X], DIMENSION_X, worldCellType, getProcessNeighbor(-1), 103, MPI_COMM_WORLD);
   }*/
}

// i = -1: last, i=+1: next
int getProcessNeighbor(int i)
{
    int neighbor=rank+i;
    if (neighbor>=totalNodes) neighbor=0;
    else if (neighbor<0) neighbor=totalNodes-1;
    return neighbor;
}

void initializeParallelEnviroment(int argc,char** argv){
    MPI_Init(&argc,&argv);
    MPI_Comm_size(MPI_COMM_WORLD, &totalNodes); // get totalnodes
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);

    // Welt wird in Blöcke von Zeilen aufgeteilt
    initializeWorld(DIMENSION_X,DIMENSION_Y/totalNodes+2);
    cells = DIMENSION_X*(DIMENSION_Y/totalNodes);

    if (rank==0) fullWorld=(WorldCell*)malloc(sizeof(WorldCell)*DIMENSION_X*DIMENSION_Y);
   
    neighbor1=getProcessNeighbor(1);
    neighbor2=getProcessNeighbor(-1);


    //WorldCell Type deklarieren
    MPI_Aint disp[3];
    MPI_Datatype type[3] = { MPI_DOUBLE, MPI_DOUBLE, MPI_INT };
    int blocklen[3] = { 1, 1, 1 };


    disp[0]=(void*)&world[0].T-(void*)&world[0];
    disp[1]=(void*)&world[0].A-(void*)&world[0];
    disp[2]=(void*)&world[0].alive-(void*)&world[0];
    MPI_Type_create_struct(3, blocklen, disp, type, &worldCellType);
    MPI_Type_commit(&worldCellType);
    srand(rank);
}
void finalizeParallelEnviroment(){
    MPI_Finalize();
}


