//
//  functions.h
//  Daisyworld
//
//  Created by Christoph Bockhahn & Max Hänze on 26.06.12.
//  Copyright (c) 2012 /. All rights reserved.
//

#ifndef Daisyworld_functions_h
#define Daisyworld_functions_h

#include "paths.h"

/*
 Modell Variablen durch Preprocessor festsetzen
 */
// Constants (paper VON BLOH ET AL.)
#define BOLZ 5.6696e-8// Bolzmann Faktor
#define TOPT 295.5
#define GAMMA 0.02
#define DT 500.0
#define C 2500.0
#define A_0 0.5
#define DELTA_T 1.0
#define DELTA_X 1.0
#define DELTA_Y 1.0
#define MUTATION_RATE 0.01

#define DIMENSION_X 480
#define DIMENSION_Y 480

// Temperaturen in Kelvin!

// Albedos
#define ALBEDOSURFACE 0.5
#define ALBEDOWHITE 0.8
#define ALBEDOBLACK 0.2

// und so weiter...

typedef struct {
    double T; // Temperatur
    double A; // Albedo
    int alive; // 0 = dead, 1 = alive , function c(x,y,t) im Paper
}WorldCell;

typedef struct {
    double* a;
    double* b;
    double* c;
} TridiagonalMatrix;

typedef struct {
    double * delta;
    double * gamma;
    double * alpha;
    double * c;
    double * z;
} SolverVariables;


//Enum type für getNeighbour Funktion
enum neighbour { up=0, down=1, left=2, right=3 };


// Array aus Rechenzellen -> unsere Welt
WorldCell* world;
WorldCell* newWorld;

TridiagonalMatrix MatrixA;

SolverVariables solver;

double* VectorU;
double* VectorB;

double luminosity;
int timestep;

double normS;
double alpha ;

int dumpStep;

//main.c
int initializeWorld (int x,int y);
//int eulerSolveStepTwo (WorldCell* cell,double dt);

//tridiagonal_cyclic_solver
void gsl_cyclic_tridiagonal( double alpha, double* u, double* b, int N);

//helper_functions.c
double rand_intervall(double a, double b);

//drawImage.cpp
void drawImageFromArray(WorldCell* world,int timestepToDraw);
void writeDataToTXT(int step, double temperature, double lum);

// NEW
void initializeMatrixA();
void initializeVectorU();
void initializeVectorB();
void initializeSolver();

void solveTimestep();

void solveT_FTCS();
void solveT();
void solveA();

void computeVectorB_l(int l);
void computeVectorB_j(int j);

double normStoS();

int getNeighbour(int index, enum neighbour nextNeighbour);

void solveCyclicTridiagonal();
void solve_cyc_tridiag(const double diag[],
                       const double offdiag[],
                       const double b[],
                       double x[],
                       int N);



void writeVectorU_ltoNewWorldCell(int l);
void writeVectorU_jtoNewWorldCell(int j);

void applyDeathRate(int i);
void applyBirthRate(int i);


void swapWorlds();

int getRandomNextNeighbour(int i);
double beta (double T);
double mutationFunction(double A);

void deallocateAll();

void writePropertiesToFile();
void writeWorldToBinaryFile(WorldCell* worldArray);

#endif
