//
//  helper_functions.c
//  Daisyworld
//
//  Created by Christoph Bockhahn & Max Hänze on 26.06.12.
//  Copyright (c) 2012 /. All rights reserved.
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "declarations.h"
#include <string.h>



int getNeighbour(int index, enum neighbour nextNeighbour)
{
    
    switch (nextNeighbour) {
        case left:
            if(index % DIMENSION_X == 0)  //Zelle liegt am linken Rand --> Zelle am rechten Rand in der selben Zeile zurück geben
            {
                return (index+DIMENSION_X-1); 
            }            
            else 
            {
                return (index-1);
            }
            break;
        case right:
            if((index+1) % DIMENSION_X == 0)  //Zelle liegt am rechten Rand --> Zelle am linken Rand in der selben Zeile zurück geben
            {
                return (index-DIMENSION_X+1); 
            }            
            else 
            {
                return (index+1);
            }
            break;
        case down:
            if((index+DIMENSION_X) >= (DIMENSION_X*DIMENSION_Y)) // Zelle liegt am unteren Rand
            {
                return (index+DIMENSION_X-(DIMENSION_X*DIMENSION_X));
            }
            else 
            {
                return (index+DIMENSION_X);
            }
            break;
        case up:
            if((index-DIMENSION_X) < 0) // Zelle liegt am oberen Rand
            {
                return (index-DIMENSION_X+(DIMENSION_X*DIMENSION_X));
            }
            else 
            {
                return (index-DIMENSION_X);
            }
            break;
    }    
    return 0;
}

//gibt eine Zufallszahl im Intervall [a,b] zurück
double rand_intervall(double a, double b)
{
    return (b-a)*(rand() / (double) RAND_MAX ) + a;
}

int getRandomNextNeighbour(int i)
{
    int randomNumber;
    randomNumber = rand() % 4; //random number between 0 and 4
    return getNeighbour(i, randomNumber);
}

void initializeMatrixA()
{
    
    MatrixA.a = (double*)malloc(sizeof(double)*DIMENSION_X);
    MatrixA.b = (double*)malloc(sizeof(double)*DIMENSION_X);
    MatrixA.c = (double*)malloc(sizeof(double)*DIMENSION_X);
    for (int i=0;i<DIMENSION_X;i++)
    {
        MatrixA.b[i]=1+alpha;
        MatrixA.a[i]=-alpha*0.5;
        MatrixA.c[i]=-alpha*0.5;
    }
    
    return;
}
void initializeVectorU()
{
    VectorU = (double*) malloc(sizeof(double)*DIMENSION_X);
    return;
}
void initializeVectorB()
{
    VectorB = (double*) malloc(sizeof(double)*DIMENSION_X);
    return;
}

void initializeSolver()
{
    int N=DIMENSION_X;
    solver.delta = (double *) malloc (N * sizeof (double));
    solver.gamma = (double *) malloc (N * sizeof (double));
    solver.alpha = (double *) malloc (N * sizeof (double));
    solver.c = (double *) malloc (N * sizeof (double));
    solver.z = (double *) malloc (N * sizeof (double));
}

void solveTimestep()
{
    solveT_FTCS();
    //solveT();
    solveA();
    return;
}

void solveT()
{
    for(int l=0;l<DIMENSION_Y;l++)
    {
        computeVectorB_l(l);
        solveCyclicTridiagonal();
        writeVectorU_ltoNewWorldCell(l);
    }
    swapWorlds(); 
    for(int j=0;j<DIMENSION_X;j++)
    {
        computeVectorB_j(j);
        solveCyclicTridiagonal();
        writeVectorU_jtoNewWorldCell(j);
    }
    swapWorlds();
}

void solveT_FTCS() 
{
//
//
    for (int i=0; i<DIMENSION_X*DIMENSION_Y;i++)
    {
        newWorld[i].T=alpha*(world[getNeighbour(i,up)].T+world[getNeighbour(i,down)].T+world[getNeighbour(i,left)].T+world[getNeighbour(i,right)].T-4*world[i].T);
        newWorld[i].T+=1.0/C*DELTA_T*(normStoS()*(1-world[i].A)-BOLZ*pow(world[i].T,4) );
        newWorld[i].T+=world[i].T;
        newWorld[i].A=world[i].A;
        newWorld[i].alive=world[i].alive;
    }

    swapWorlds();
}

void solveA()
{
    //iteriere durch alle Zellen der WorldCell
    for(int i=0;i<DIMENSION_X*DIMENSION_Y;i++)

    {
            if(world[i].alive == 1) //Zelle lebt
            {
                applyDeathRate(i);
            }
            else { // Zelle lebt nicht
                applyBirthRate(i); 
            }

            newWorld[i].T=world[i].T;
    }
    swapWorlds();
    return;
}

void computeVectorB_l(int l)
{
    for(int j=0;j<DIMENSION_X;j++)
    {
        VectorB[j] = alpha/2.0 * world[getNeighbour(j+l*DIMENSION_X,up)].T ;
        VectorB[j] += (1.0-alpha) * world[j+l*DIMENSION_X].T;
        VectorB[j] +=alpha/2.0 * world[getNeighbour(j+l*DIMENSION_X,down)].T ;
        VectorB[j] += 0.5/C*DELTA_T*(normStoS()*(1-world[j+l*DIMENSION_X].A)-BOLZ*pow(world[j+l*DIMENSION_X].T,4) );
    }
  
    return;
}



void computeVectorB_j(int j)
{
    for (int l=0; l<DIMENSION_Y; l++)
    {
        VectorB[l]=alpha/2.0 * world[getNeighbour(j+DIMENSION_X*l,left)].T;
        VectorB[l]+=(1-alpha)* world[j+DIMENSION_X*l].T;
        VectorB[l]+=alpha/2.0 * world[getNeighbour(j+DIMENSION_X*l,right)].T;
        VectorB[l]+=0.5*DELTA_T/C* (normStoS() *(1-world[j+DIMENSION_X*l].A)-BOLZ*pow(world[j+DIMENSION_X*l].T,4) );
        
    }
    return;
}

double normStoS()
{
    
    return BOLZ*pow(TOPT,4) / (1.0-A_0)*normS;
}

void swapWorlds()
{
    WorldCell* tmp=world;
    world=newWorld;
    newWorld=tmp;

};

void writeVectorU_ltoNewWorldCell(int l)
{
    for(int j=0;j<DIMENSION_X;j++)
    {
        newWorld[j+l*DIMENSION_X].T = VectorU[j];
    }
    return;
}

void solveCyclicTridiagonal()
{
    //tridiag(DIMENSION_X, Ma,  MatrixA->b,  MatrixA->c, VectorB, VectorU);
    //tridiag(DIMENSION_X, MatrixA.a,MatrixA.b,MatrixA.c, VectorB, VectorU);
    solve_cyc_tridiag(MatrixA.b,MatrixA.a,VectorB,VectorU,DIMENSION_X);
    //gsl_cyclic_tridiagonal(alpha, VectorU, VectorB, DIMENSION_X);
    return;
}


void writeVectorU_jtoNewWorldCell(int j)
{
    for (int l=0;l< DIMENSION_Y; l++)
    {
        newWorld[j+DIMENSION_X*l].T=VectorU[l];
    }
    return;
}


void applyBirthRate(int i)
{
    
    double randomNumber;
    //choose random next neighbour
    int nextNeighbour = getRandomNextNeighbour(i);
    
    if(world[nextNeighbour].alive == 1)
    {                
        //Zelle wächst mit Wsk Beta
        randomNumber = rand_intervall(0,1);  
        if(randomNumber <= beta(world[nextNeighbour].T))
        {
            newWorld[i].alive = 1;
            // Albedo wird von der zufälligen Nachbarzelle übernommen und unterliegt einer Mutationsrate
            newWorld[i].A = mutationFunction(world[nextNeighbour].A);    
        }
    }
    else 
    {
        //Zelle wächst nicht
        newWorld[i].alive = 0; 
        newWorld[i].A = A_0;
    }
}

void applyDeathRate(int i)
{
    double randomNumber = rand_intervall(0,1);
    if(randomNumber < GAMMA)
    {
        newWorld[i].alive = 0; // Zelle stirbt mit Wsk 1-GAMMMA
        newWorld[i].A = A_0; //setze Albedo zurück auf Ausgangswert A_0
        //printf("STIRB! \n");
    }
    else {
        newWorld[i].alive = world[i].alive;
        newWorld[i].A = world[i].A;
    }
}


double mutationFunction(double A)
{
    double result = A + rand_intervall(-MUTATION_RATE, MUTATION_RATE);
    if (result < 0 ) result = 0;
    else if (result > 1) result = 1;
    
    return result;
}


void writeWorldToBinaryFile(WorldCell* worldArray)
{
    char outputName[1000];
    sprintf(outputName,"%sWorld_%d.bin",PATH,dumpStep);
    FILE* file=fopen(outputName,"wb");
    int nb=DIMENSION_X;
    fwrite(&nb,1,4,file);
    fwrite(worldArray,sizeof(WorldCell),DIMENSION_X*DIMENSION_Y,file);
    
    fclose(file);
    dumpStep++;
}

void writePropertiesToFile(){
    static int firstRun = 1;
   //Ergebnisse ausgeben
    double sum_T=0l;
    double sum_albedo=0l;
    int alive=0;
    double N=DIMENSION_X*DIMENSION_Y;
    for(int i=0;i<N;i++)
    {
        sum_T += world[i].T;
        sum_albedo += world[i].A;
        alive += world[i].alive;
    }
    char outputName[1000];
    sprintf(outputName,"%s%s",PATH,"stats.txt");
    FILE* file;
    if (firstRun==1){
        file=fopen(outputName, "w");
        firstRun=0;
    }
    else file=fopen(outputName,"a"); 
    
    fprintf(file,"%le %le %le %d\n", normS,sum_T/N,sum_albedo/N,alive);
    fclose(file);
    return;


}

void deallocateAll(){
    free(world);
    free(newWorld);
    free(MatrixA.a);
    free(MatrixA.b);
    free(MatrixA.c);
    free(VectorU);
    free(VectorB);
    
    free(solver.alpha);
    free(solver.c);
    free(solver.delta);
    free(solver.gamma);
    free(solver.z);
}



