//
//  main.c
//  Daisyworld
//
//  Created by Christoph Bockhahn & Max Hänze on 24.05.12.
//  Copyright (c) 2012 /. All rights reserved.
//


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "declarations.h"

// auskommentieren falls kein Debug
#define DEBUG_LOG

/*
   Einzelne Rechenzelle mit allen nötigen Informationen
*/



/*
 Initialisiere Welt mit Anzahl x*y von Zellen
 und Anfangswerten aus initCell
 */

int initializeWorld (int x,int y) {
    dumpStep=0;

    alpha = DT/C * DELTA_T / pow(DELTA_X,2);
    normS = 1.0;
    //printf("ALPHA: %f \n \n \n",alpha);
    // Allocate memory
    world =(WorldCell*) malloc(sizeof(WorldCell)*x*y);
    newWorld = (WorldCell*)malloc(sizeof(WorldCell)*x*y);   
    // Alle Zellen auf initCell setzen
    for (int i=0; i<x*y;i++){
        world[i].T = TOPT;
        world[i].alive = rand() % 2;
        world[i].A = rand() / (double)RAND_MAX;
    }
#ifdef DEBUG_LOG
    // gebe Gesamtspeicher der Welt aus
    printf("Memory used by world: %.1f MByte \n", 2.0f*sizeof(WorldCell)*x*y/1000000.f);
#endif
    
    return 0;
}


int main(int argc, const char * argv[])
{
    int N = DIMENSION_X*DIMENSION_Y;
    
    initializeWorld(DIMENSION_X,DIMENSION_Y);    
    initializeMatrixA();
    initializeVectorU();
    initializeVectorB();
    initializeSolver();
   
    // Gleichgewichtszustand finden
    for(int steps=0;steps<1e4;steps++)
    {
        solveTimestep();
    }


    for(int steps=0;steps<2e5;steps++)
    {
        solveTimestep();
        
        
        if (steps%223==0) writePropertiesToFile();               
        normS+=1e-5;
     
        //printf("Schritt: %d \n",steps);        
    }
        
    
    //Ergebnisse ausgeben
    double sum_T=0l;
    double sum_albedo=0l;
    int alive=0;
    for(int i=0;i<N;i++)
    {
        //printf("T(%d) = %f - Albedo: %f - Alive: %d \n",i,world[i].T,world[i].A,world[i].alive);
        sum_T += world[i].T;
        sum_albedo += world[i].A;
        alive += world[i].alive;
    }

    printf("Mean Temperature: %f \t Mean Albedo: %f \t %d von %d Daisies leben \n \n",sum_T/N,sum_albedo/N,alive,N);
    //Bild ausgeben
    //writeWorldToBinaryFile(world);       
    deallocateAll();
    
    return 0;    
}

