//
//  math_functions.c
//  Daisyworld
//
//  Created by Christoph Bockhahn & Max Hänze on 26.06.12.
//  Copyright (c) 2012 /. All rights reserved.
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "declarations.h"

// Beta ist die Wachstumsrate der Gänseblümchen bei gegebener Temperatur
// durch inline wird code aus Funktion direkt an der ausführenden stelle platziert
double beta (double T) {
    static const double c = 4.0/35.0/35.0;   
    double result = c * (T-278)*(313-T);
    if (result > 0) return result;
    else return 0;
}

void solve_cyc_tridiag(
                  const double diag[],
                  const double offdiag[],
                  const double b[],
                  double x[],
                  int N)
{
    double * delta= solver.delta;
    double * gamma = solver.gamma;
    double * alpha = solver.alpha;
    double * c = solver.c;
    double * z = solver.z;
    
    if (delta == 0 || gamma == 0 || alpha == 0 || c == 0 || z == 0)
    {
        printf("failed to allocate working space");
        return;
    }
    else
    {
        int i, j;
        double sum = 0.0;
        
        /* factor */
        
        if (N == 1) 
        {
            x[0] = b[0] / diag[0];
        }
        
        alpha[0] = diag[0];
        gamma[0] = offdiag[0] / alpha[0];
        delta[0] = offdiag[(N-1)] / alpha[0];
        
        if (alpha[0] == 0) {
            printf("ERROR: alpha=0\n");
            return;
        }
        
        for (i = 1; i < N - 2; i++)
        {
            alpha[i] = diag[ i] - offdiag[(i-1)] * gamma[i - 1];
            gamma[i] = offdiag[ i] / alpha[i];
            delta[i] = -delta[i - 1] * offdiag[(i-1)] / alpha[i];
            if (alpha[i] == 0) {
                printf("ERROR: alpha=0\n");
                return;
            }
        }
        
        for (i = 0; i < N - 2; i++)
        {
            sum += alpha[i] * delta[i] * delta[i];
        }
        
        alpha[N - 2] = diag[(N - 2)] - offdiag[(N - 3)] * gamma[N - 3];
        
        gamma[N - 2] = (offdiag[(N - 2)] - offdiag[(N - 3)] * delta[N - 3]) / alpha[N - 2];
        
        alpha[N - 1] = diag[(N - 1)] - sum - alpha[(N - 2)] * gamma[N - 2] * gamma[N - 2];
        
        /* update */
        z[0] = b[0];
        for (i = 1; i < N - 1; i++)
        {
            z[i] = b[i] - z[i - 1] * gamma[i - 1];
        }
        sum = 0.0;
        for (i = 0; i < N - 2; i++)
        {
            sum += delta[i] * z[i];
        }
        z[N - 1] = b[(N - 1)] - sum - gamma[N - 2] * z[N - 2];
        for (i = 0; i < N; i++)
        {
            c[i] = z[i] / alpha[i];
        }
        
        /* backsubstitution */
        x[(N - 1)] = c[N - 1];
        x[(N - 2)] = c[N - 2] - gamma[N - 2] * x[(N - 1)];
        if (N >= 3)
        {
            for (i = N - 3, j = 0; j <= N - 3; j++, i--)
            {
                x[i] = c[i] - gamma[i] * x[(i + 1)] - delta[i] * x[(N - 1)];
            }
        }
    }
    
    return;
}



