//
//  main.cpp
//  DaisyworldImage
//
//  Copyright (c) 2012 /. All rights reserved.
//

#include <iostream>
#include <Magick++.h>
#include <sstream>
#include <fstream>
#include <cmath>
#include "declarations.h"
#include "path.h"

using namespace std;
using namespace Magick;

template <typename T>
string NumberToString ( T Number )
{
    ostringstream ss;
    ss << Number;
    return ss.str();
}

int main(int argc, const char * argv[])
{    
     InitializeMagick(*argv);
    
    //get N
    int N=getInitialN();
    
    world = new WorldCell[N*N];
    
    for(int i=0;i<1000;i++)
    {
        readWorld(i);
        drawImageFromArray(world,i,N,N);
    }
    
   
    
    
    return 0;
}

void readWorld(int index)
{
    int N = 0;
    FILE * pFile;
    long lSize;
    size_t result;
    
    string filePathString = PATH;    
    string tempPath = filePathString + "visualisation_World_" + NumberToString(index) + ".bin";

    //cast string to char*
    char* filePath = const_cast<char*>(tempPath.c_str());
    
    pFile = fopen ( filePath , "rb" );
    if (pFile==NULL) {fputs ("End of file list reached or can't read binary file.",stderr); exit (1);}
    
    // get file size
    fseek (pFile , 0 , SEEK_END);
    lSize = ftell (pFile);
    rewind (pFile);
    
    // copy the file contents into the buffer
    result = fread(&N,1,4,pFile);
    
    result = fread (world,1,N*N*sizeof(WorldCell),pFile);
   
    // close file and free memory
    fclose (pFile);

    return;
}

int getInitialN()
{
    int result;
    FILE * pFile;
    
    string filePathString = PATH;    
    string tempPath = filePathString + "visualisation_World_0.bin";
    char* filePath = const_cast<char*>(tempPath.c_str());
    pFile = fopen ( filePath , "rb" );
    if (pFile==NULL) {fputs ("Cant read binary file...",stderr); exit (1);}    
    fread(&result,1,4,pFile);
    fclose(pFile);    
    return result;
}


double red(double x)
{
    double val=0.;
    
    if(x < 0.38) 
    { 
        val = 0.; 
    }
    else if(x >= 0.38 && x < 0.62) 
    { 
        val = ((x-0.38)*4.17); 
    }
    else if(x >= 0.62 && x < 0.88) 
    {
        val = 1.;
    }
    else if(x >= 0.88 && x <= 1.)
    {
        val = (1-(x-0.88)*4.17);
    }
    else 
    {
        val = 0.5;
    }
    
    if(val > 1) return 1.;
    if(val < 0) return 0.;
    return val;
}


double green(double x)
{
    double val=0.;
    
    if(x < 0.12) 
    { 
        val = 0.; 
    }
    else if(x >= 0.12 && x < 0.38) 
    { 
        val = ((x-0.12)*4.17);
    }
    else if(x >= 0.38 && x < 0.62) 
    {
        val = 1.;
    }
    else if(x >= 0.62 && x < 0.88)
    {
        val = (1-(x-0.62)*4.17);
    }
    else 
    {
        val = 0.;
    }
    
    if(val > 1) return 1.;
    if(val < 0) return 0.;
    return val;
}

double blue(double x)
{
    
    double val=0.;
    
    if(x < 0)
    {
        val = 0.5;
    }
    else if(x < 0.12) 
    { 
        val = (0.5+x*4.17); 
    }
    else if(x >= 0.12 && x < 0.38) 
    { 
        val = 1.; 
    }
    else if(x >= 0.38 && x <= 0.62) 
    {
        val = (1-(x-0.38)*4.17);
    }
    else 
    {
        val = 0.;
    }
    
    if(val > 1) return 1.;
    if(val < 0) return 0.;
    return val;
}




void drawImageFromArray(WorldCell* world,int timestepToDraw,int dimensionX, int dimensionY)
{
    
    int stepsize = 500; //number of different colors to be drawn
    
    int minimum = world[0].T;
    int maximum = world[0].T;
    
    //find minimum and maximum temperature in world array
    for(int i=0;i<dimensionX*dimensionY;i++)
    {
        if(world[i].T > maximum) maximum=world[i].T;
        if(world[i].T < minimum) minimum=world[i].T;
    }
    
    if(minimum == maximum) printf("Warning! Min/Max Temperature are equal!"); maximum++;
    
    minimum=278;
    maximum=313;
    
    //minimum=283;
    //maximum=308;
    
    //printf("min/max: %d %d \n",minimum,maximum);
    
    //albedo distribution (grayscale)
    Image albedo_distribution( Geometry(dimensionX, dimensionY), Color(MaxRGB, MaxRGB, MaxRGB, 0));
    
    //temperature distribution (color gradient)
    Image temperature_distribution( Geometry(dimensionX, dimensionY), Color(MaxRGB, MaxRGB, MaxRGB, 0));
    
    //scale for temperature distribution
    Image temperateScale(Geometry(500,500),Color(MaxRGB,MaxRGB,MaxRGB,0)); 
    temperateScale.type(TrueColorType);
    
    for(int i=0;i<dimensionX;i++)
    {
        for(int j=0;j<dimensionY;j++)
        {
            
            
            
            double albedo = world[i+j*dimensionY].A;
            Color colorToDraw = Color(MaxRGB * 0.7 * albedo,MaxRGB*albedo,MaxRGB*0.6*albedo,0);
            
            
            if(world[i+j*dimensionY].alive == 0) 
            {
                 colorToDraw = Color(MaxRGB * 193. / 255,MaxRGB * 173 / 255., MaxRGB * 59. / 255);
            }
            

            albedo_distribution.pixelColor(i,j,colorToDraw);
            
            double temperature = world[i+j*dimensionY].T;
            
            double factor = (temperature - minimum) / (maximum - minimum);            
            Quantum R = MaxRGB*red(factor);
            Quantum G = MaxRGB*green(factor);
            Quantum B = MaxRGB*blue(factor);
            
            
            colorToDraw = Color(R,G,B,0);
            
            temperature_distribution.pixelColor(i,j,colorToDraw);
        }
    }
    
    double factor = 0.;
    
    //draw 100 lines for color gradient
    for(int i=0;i<stepsize;i++)
    {
        //interpolate between blue (cold) and red (hot)
        
        Color colorToDraw = Color(MaxRGB*red(factor),MaxRGB*green(factor),MaxRGB*blue(factor));
        
        factor += 0.002;
        
        //scale is 250px wide
        for(int j=0;j<250;j++)
        {            
            //draw pixels to image
            temperateScale.pixelColor(j,i,colorToDraw);
        }
    }    
    
    albedo_distribution.magick("png");  
    temperature_distribution.magick("png");
    temperateScale.magick("png");
    
    string filepath = PATH;
    string strAlbedo_distribution = filepath + "albedo" + NumberToString(timestepToDraw) + ".png";
    string strTemperatureDistribution = filepath + "temperature" + NumberToString(timestepToDraw) + ".png";
    
    albedo_distribution.write(strAlbedo_distribution); 
    temperature_distribution.write(strTemperatureDistribution); 
    temperateScale.write(filepath + "temperature_scale.png");
    
    return;
}

